//
// C++ Implementation: cgaugebaritem
//
// Description: 
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "cgaugebaritem.h"

#include "cgauge.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>

cGaugeBarItem::cGaugeBarItem (cGauge *g, QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  QHBoxLayout *layout = new QHBoxLayout (this);
  layout->setAutoAdd (true);
  layout->setSpacing (2);
  label = new QLabel (this);
  gaugepriv = new cGaugeBarItemPrivate (this);
  gauge = g;
}

cGaugeBarItem::~cGaugeBarItem ()
{
}

void cGaugeBarItem::setText (const QString &caption)
{
  label->setText (caption);
}

QString cGaugeBarItem::text () const
{
  return label->text();
}

void cGaugeBarItem::setValue (int value)
{
  gaugepriv->setValue (value);
}

int cGaugeBarItem::value () const
{
  return gaugepriv->value();
}

void cGaugeBarItem::updateValue ()
{
  //hidden?
  bool hidden = gauge->hidden();
  hidden ? hide() : show();

  //color
  gaugepriv->setColor (gauge->color());

  //text
  setText (gauge->caption());

  //value
  int val = gauge->varValue();
  int maxval = 100;
  QString maxvar = gauge->maxVariable();
  if (!maxvar.isEmpty())
    maxval = gauge->maxVarValue();
  if (maxval == 0)
    maxval = 100;
  setValue (val * 100 / maxval);
}

cGaugeBarItemPrivate::cGaugeBarItemPrivate (QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  val = 0;
  col = white;

  setMaximumSize (102, 14);
  setMinimumSize (22, 14);
  setBaseSize (102, 14);
  setFocusPolicy (QWidget::NoFocus);
  setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed);
}

cGaugeBarItemPrivate::~cGaugeBarItemPrivate ()
{

}

void cGaugeBarItemPrivate::setValue (int value)
{
  val = value;
  update ();
}

int cGaugeBarItemPrivate::value () const
{
  return val;
}

void cGaugeBarItemPrivate::setColor (const QColor &color)
{
  col = color;
  update ();
}

QColor cGaugeBarItemPrivate::color () const
{
  return col;
}

void cGaugeBarItemPrivate::paintEvent (QPaintEvent *)
{
  int w = width();
  int h = height();
  QPainter p (this);
  p.setPen (colorGroup().dark());
  p.drawRect (0, 0, w, h);
  if (val)
  {
    w = val * (w - 2) / 100;   //width of the colored part
    p.fillRect (1, 1, w, h - 2, col);
  }
}

QSize cGaugeBarItemPrivate::sizeHint () const
{
  return QSize (102, 14);
}

#include "cgaugebaritem.moc"
