//
// C++ Implementation: cgaugelist
//
// Description: 
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cgaugelist.h"

#include "cgauge.h"
#include "cgaugebar.h"
#include "kmuddy.h"

#include <kconfig.h>

extern KMuddy *kmuddy;

cGaugeList::cGaugeList (QString filename, int sess) :
    cSaveableList ("gaugelist", sess)
{
  prototype = new cGauge (sess);
  objName = "Gauge";
  config = new KConfig (filename);
  load ();

  gaugesChanged ();

  //show the gaugebar is we have some unhidden item
  bool shouldShow = false;
  for (reset(); *this; (*this)++)
  {
    cGauge *gauge = (cGauge *) **this;
    if (!gauge->hidden())
    {
      shouldShow = true;
      break;
    }
  }
  if (shouldShow)
    kmuddy->showGauges (true);

  addEventHandler ("var-changed", 500, PT_STRING);
}

cGaugeList::~cGaugeList()
{
  removeEventHandler ("var-changed");
  save ();
  config->sync ();
}

void cGaugeList::eventHandler (QString event, int session,
    QString &par1, const QString &par2)
{
  if (event == "var-changed")
    variableChanged (par1, par2);
}

bool cGaugeList::requestGauge (const QString &var, const QString &maxvar, const QString &caption,
    const QColor &color, bool hidden)
{
  for (reset(); *this; (*this)++)
    if (var == ((cGauge *) cur)->variable())  //variable already in use
      return false;
  
  //OKay, variable not yet in use
  cGauge *gauge = new cGauge (sess());
  gauge->setVariable (var);
  gauge->setMaxVariable (maxvar);
  gauge->setCaption (caption);
  gauge->setColor (color);
  gauge->setHidden (hidden);
  addToEnd (gauge);
  gaugesChanged ();
  
  //show the gauge bar if it was hidden previously
  if (!hidden)
    kmuddy->showGauges (true);
  
  return true;
}

bool cGaugeList::requestGauge (cGauge *gauge)
{
  QString var = gauge->variable();
  for (reset(); *this; (*this)++)
    if (var == ((cGauge *) cur)->variable())  //variable already in use
    {
      delete gauge;  //delete the gauge
      return false;
    }
  
  addToEnd (gauge);
  gaugesChanged ();
  
  //show the gauge bar if it was hidden previously
  kmuddy->showGauges (true);
  
  return true;
}

void cGaugeList::gaugesChanged ()
{
  (dynamic_cast<cGaugeBar *>(object ("gaugebar")))->rebuild (this);
}

void cGaugeList::variableChanged (const QString &varname, const QString &value)
{
  bool ok;
  int val = value.toInt (&ok);
  val = ok ? val : 0;
  
  //look if some gauge uses the changed variable
  for (reset(); *this; (*this)++)
    ((cGauge *) cur)->varValueChanged (varname, val);
}


#include "cgaugelist.moc"
