/***************************************************************************
                          cglobalsettings.cpp  -  global application settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : St Aug 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cglobalsettings.h"

#include "dialogs/dlgappsettings.h"
#include "cactionmanager.h"
#include "cansiparser.h"
#include "ccmdparser.h"
#include "ccmdprocessor.h"
#include "cconsole.h"
#include "coutput.h"
#include "cinputline.h"
#include "cmultiinputline.h"
#include "cmultilineinput.h"
#include "cpluginmanager.h"
#include "csession.h"
#include "csessionmanager.h"
#include "ctelnet.h"
#include "cconnprefs.h"
#include "cwindowlist.h"
#include "kmuddy.h"

#ifdef HAVE_MXP
#include "cmxpmanager.h"
#endif

#include <kapp.h>
#include <kconfig.h>
#include <kaction.h>

extern KMuddy *kmuddy;
extern cMultiLineInput *multiline;

cGlobalSettings::cGlobalSettings() : QObject ()
{
  load ();
}

cGlobalSettings::~cGlobalSettings()
{
  save ();
}

void cGlobalSettings::load ()
{
  //Loads settings from kmuddyrc. Defaults are used if necessary.
  KConfig *config = (KApplication::kApplication())->config ();

  //reparse config configuration - needed to make updating of settings
  //via IPC work
  config->reparseConfiguration ();
  config->setReadOnly (false);
  
  //Window
  config->setGroup ("Window");
  alwaystabbar = config->readBoolEntry ("Always show tab bar", true);
  auxinput = config->readBoolEntry ("Auxiliary input", false);
  globalnotify = config->readBoolEntry ("Global notification", true);
  localnotify = config->readBoolEntry ("Local notification", true);
  alwaysnotify = config->readBoolEntry ("Always notify", true);
  autoconnect = config->readEntry ("Auto connect", QString::null);
  //all these are kept in Window group for historical reasons
  // (it used to be there in 0.3.1 and before :))
  bkcolorindex = config->readNumEntry ("Background color index", CL_BLACK);
  fgcolorindex = config->readNumEntry ("Foreground color index", CL_WHITE);
  echoindex = config->readNumEntry ("Echo color index", CL_YELLOW | CL_BRIGHT);
  systemindex = config->readNumEntry ("System color index", CL_CYAN | CL_BRIGHT);
  cmdecho = config->readBoolEntry ("Command echo", true);
  lpmudstyle = config->readBoolEntry("LPMud style", false);
  messages = config->readBoolEntry ("Display messages", true);
  allowblinking = config->readBoolEntry ("Allow blinking", false);
  wordwrapping = config->readBoolEntry ("Word wrapping", true);
  indentvalue = config->readNumEntry ("Indentation", 0);
  wrappos = config->readNumEntry ("Wrap position", 0);
  hissize = config->readNumEntry ("History size", 1000);
  forceredraw = config->readNumEntry ("Force redraw", 10);
  
  //Fonts
  config->setGroup ("Fonts");
  QFont f = KGlobalSettings::fixedFont ();
  cfont = config->readFontEntry ("Console font", &f);
  ifont = config->readFontEntry ("Input line font", &f);
  mfont = config->readFontEntry ("Multi-line input font", &f);

  //Input line
  config->setGroup ("Input line");
  keeptext = config->readBoolEntry ("Keep text", true);
  selectkepttext = config->readBoolEntry ("Select kept text", true);
  arrowshistory = config->readBoolEntry ("Cursor keys browse history", true);
  useac = config->readBoolEntry ("Use auto-completion", true);
  typeofac = config->readNumEntry ("Auto-completion type", 5);
  tnpaste = config->readBoolEntry ("Telnet-style paste", false);
  trimspaces = config->readBoolEntry ("Trim spaces", false);
  inputbkcolorindex = config->readNumEntry ("Background color", CL_BLACK);
  inputcolorindex = config->readNumEntry ("Text color", CL_YELLOW | CL_BRIGHT);
  swapenters = config->readBoolEntry ("Swap ENTERs", false);

  config->setGroup ("Colors");
  color[CL_BLACK] = config->readColorEntry ("Black", &black);
  color[CL_RED] = config->readColorEntry ("Red", &darkRed);
  color[CL_GREEN] = config->readColorEntry ("Green", &darkGreen);
  color[CL_YELLOW] = config->readColorEntry ("Yellow", &darkYellow);
  color[CL_BLUE] = config->readColorEntry ("Blue", &darkBlue);
  color[CL_MAGENTA] = config->readColorEntry ("Magenta", &darkMagenta);
  color[CL_CYAN] = config->readColorEntry ("Cyan", &darkCyan);
  color[CL_WHITE] = config->readColorEntry ("Gray", &lightGray);
  color[CL_BLACK | CL_BRIGHT] = config->readColorEntry ("Dark gray", &darkGray);
  color[CL_RED | CL_BRIGHT] = config->readColorEntry ("Bright red", &red);
  color[CL_GREEN | CL_BRIGHT] = config->readColorEntry ("Bright green", &green);
  color[CL_YELLOW | CL_BRIGHT] = config->readColorEntry ("Bright yellow", &yellow);
  color[CL_BLUE | CL_BRIGHT] = config->readColorEntry ("Bright blue", &blue);
  color[CL_MAGENTA | CL_BRIGHT] = config->readColorEntry ("Bright magenta", &magenta);
  color[CL_CYAN | CL_BRIGHT] = config->readColorEntry ("Bright cyan", &cyan);
  color[CL_WHITE | CL_BRIGHT] = config->readColorEntry ("White", &white);

  //Characters (these are no longer characters, name wasn't changed
  //to make upgrading easier)
  config->setGroup ("Characters");
  separstr = config->readEntry ("Command separator", ";");
  walkstr = config->readEntry ("Speed walking", ".");
  //no longer a macro character, name kept for historical reasons
  scriptstr = config->readEntry ("Macro character", "/");
  multistr = config->readEntry ("Multi-command", "#");
  focusstr = config->readEntry ("Focus command", ":");
  noparsestr = config->readEntry ("No-parse prefix", "'");
  allowemptywalkstr = config->readBoolEntry ("Empty walkstring", false);
  expandBS = config->readBoolEntry ("Expand backslashes", true);

#ifdef HAVE_SOUND
  //MSP
  config->setGroup ("Sound Protocol");
  allowmsp = config->readBoolEntry ("Allow MSP", true);
  allowdload = config->readBoolEntry ("Allow downloads", false);
  sounddirs.clear ();
  int count = config->readNumEntry ("Path count", -1);
  if (count == -1)
  {
    count = 1;
    sounddirs << "/usr/share/sounds";
  }
  else
    for (int i = 1; i <= count; i++)
      sounddirs << config->readEntry (QString("Path ") + QString::number(i),
            QString::null);
#endif  //HAVE_SOUND

  //Shortcuts
  
  // Plugins
  config->setGroup ("Plugins");
  QStringList nl;
  nl = config->readListEntry ("Do not load", nl);
  cPluginManager::self()->setNoAutoLoadList (nl);
}

void cGlobalSettings::save ()
{
  //Saves settings back to kmuddyrc.
  KConfig *config = (KApplication::kApplication())->config ();
  
  //Window
  config->setGroup ("Window");
  config->writeEntry ("Always show tab bar", alwaystabbar);
  config->writeEntry ("Auxiliary input", auxinput);
  config->writeEntry ("Global notification", globalnotify);
  config->writeEntry ("Local notification", localnotify);
  config->writeEntry ("Always notify", alwaysnotify);
  config->writeEntry ("Auto connect", autoconnect);
  //these are all kept in Window group for hist.reasons - see load()
  config->writeEntry ("Background color index", bkcolorindex);
  config->writeEntry ("Foreground color index", fgcolorindex);
  config->writeEntry ("Echo color index", echoindex);
  config->writeEntry ("System color index", systemindex);
  config->writeEntry ("Command echo", cmdecho);
  config->writeEntry ("LPMud style", lpmudstyle);
  config->writeEntry ("Display messages", messages);
  config->writeEntry ("Allow blinking", allowblinking);
  config->writeEntry ("Word wrapping", wordwrapping);
  config->writeEntry ("Wrap position", wrappos);
  config->writeEntry ("Indentation", indentvalue);
  config->writeEntry ("History size", hissize);
  config->writeEntry ("Force redraw", forceredraw);

  //Fonts
  config->setGroup ("Fonts");
  config->writeEntry ("Console font", cfont);
  config->writeEntry ("Input line font", ifont);
  config->writeEntry ("Multi-line input font", mfont);
  
  //Input line
  config->setGroup ("Input line");
  config->writeEntry ("Keep text", keeptext);
  config->writeEntry ("Select kept text", selectkepttext);
  config->writeEntry ("Cursor keys browse history", arrowshistory);
  config->writeEntry ("Use auto-completion", useac);
  config->writeEntry ("Auto-completion type", typeofac);
  config->writeEntry ("Telnet-style paste", tnpaste);
  config->writeEntry ("Trim spaces", trimspaces);
  config->writeEntry ("Background color", inputbkcolorindex);
  config->writeEntry ("Text color", inputcolorindex);
  config->writeEntry ("Swap ENTERs", swapenters);

  //Colors
  config->setGroup ("Colors");
  config->writeEntry ("Black", color[CL_BLACK]);
  config->writeEntry ("Red", color[CL_RED]);
  config->writeEntry ("Green", color[CL_GREEN]);
  config->writeEntry ("Yellow", color[CL_YELLOW]);
  config->writeEntry ("Blue", color[CL_BLUE]);
  config->writeEntry ("Magenta", color[CL_MAGENTA]);
  config->writeEntry ("Cyan", color[CL_CYAN]);
  config->writeEntry ("Gray", color[CL_WHITE]);
  config->writeEntry ("Dark gray", color[CL_BLACK | CL_BRIGHT]);
  config->writeEntry ("Bright red", color[CL_RED | CL_BRIGHT]);
  config->writeEntry ("Bright green", color[CL_GREEN | CL_BRIGHT]);
  config->writeEntry ("Bright yellow", color[CL_YELLOW | CL_BRIGHT]);
  config->writeEntry ("Bright blue", color[CL_BLUE | CL_BRIGHT]);
  config->writeEntry ("Bright magenta", color[CL_MAGENTA | CL_BRIGHT]);
  config->writeEntry ("Bright cyan", color[CL_CYAN | CL_BRIGHT]);
  config->writeEntry ("White", color[CL_WHITE | CL_BRIGHT]);
  
  //Characters (hist. reasons, see above)
  config->setGroup ("Characters");
  config->writeEntry ("Command separator", separstr);
  config->writeEntry ("Speed walking", walkstr);
  //no longer a character, name kept for historical reasons
  config->writeEntry ("Macro character", scriptstr);
  config->writeEntry ("Multi-command", multistr);
  config->writeEntry ("Focus command", focusstr);
  config->writeEntry ("No-parse prefix", noparsestr);
  config->writeEntry ("Empty walkstring", allowemptywalkstr);
  config->writeEntry ("Expand backslashes", expandBS);
  
#ifdef HAVE_SOUND
  //MSP
  config->setGroup ("Sound Protocol");
  config->writeEntry ("Allow MSP", allowmsp);
  config->writeEntry ("Allow downloads", allowdload);
  config->writeEntry ("Path count", sounddirs.count());
  QStringList::iterator it;
  int i = 0;
  for (it = sounddirs.begin(); it != sounddirs.end(); ++it)
    config->writeEntry (QString("Path ") + QString::number(++i), *it);
#endif  //HAVE_SOUND

  //Shortcuts
  KActionCollection *acol = cActionManager::self()->getACol ();
  if (!acol) cActionManager::self()->createACol (kmuddy);
  acol = cActionManager::self()->getACol ();
  acol->writeShortcutSettings ("Shortcuts");
  
  //Plugins
  config->setGroup ("Plugins");
  config->writeEntry ("Do not load", cPluginManager::self()->noAutoLoadList());
  
  config->sync ();
}

void cGlobalSettings::apply ()
{
  multiline->setFont (mfont);

  //initial history size - must be set prior to running the session loop
  cConsole::setInitialHistorySize (hissize);
  
  cActionManager *am = cActionManager::self();
  for (int sess = 1; sess <= MAXSESSIONS; sess++)
  {
    if (am->sessionExists (sess))
    {
      cOutput *output = dynamic_cast<cOutput *>(am->object ("output", sess));
      cConsole *console = output->console();
      cInputLine *inputline = dynamic_cast<cInputLine *>(am->object ("inputline", sess));
      cMultiInputLine *multiinputline = dynamic_cast<cMultiInputLine *>(am->object ("multiinputline", sess));
      cInputLine *auxinputline = dynamic_cast<cInputLine *>(am->object ("auxinputline", sess));
      cCmdProcessor *cmdproc = dynamic_cast<cCmdProcessor *>(am->object ("cmdprocessor", sess));
      cCmdParser *cmdparser = dynamic_cast<cCmdParser *>(am->object ("cmdparser", sess));
      cANSIParser *ansiparser = dynamic_cast<cANSIParser *>(am->object ("ansiparser", sess));
      cTelnet *telnet = dynamic_cast<cTelnet *>(am->object ("telnet", sess));
#ifdef HAVE_MXP
      cMXPManager *mxpmgr = dynamic_cast<cMXPManager *>(am->object ("mxpmanager", sess));
#endif
      cWindowList *wlist = dynamic_cast<cWindowList *>(am->object ("windowlist", sess));
      cSession *session = dynamic_cast<cSession *>(am->object ("session", sess));
      
      //Window
      session->setAuxInput (auxinput);

      //Fonts
      console->setFont (cfont);
      if (wlist) wlist->adjustFonts (cfont);
      inputline->setFont (ifont);
      auxinputline->setFont (ifont);
      multiinputline->setFont (ifont);

      //Output area
      ansiparser->setDefaultTextColor (ansiparser->color (fgcolorindex));
      ansiparser->setDefaultBkColor (ansiparser->color (bkcolorindex));
      output->setEchoColor (ansiparser->color (echoindex));
      output->setSystemColor (ansiparser->color (systemindex));
      output->setEnableEcho (cmdecho);
      output->setEnableMessages (messages);
      console->setEnableBlinking (allowblinking);
      console->setWordWrapping (wordwrapping);
      console->setWrapPos (wrappos);
      console->setIndentation (indentvalue);
      console->setRepaintCount (forceredraw);
      if (wlist) wlist->applySettings (allowblinking, wordwrapping, wrappos,
	  indentvalue, forceredraw);
      console->tryUpdateHistorySize ();
      //hissize and alwaystabbar are set at the begin/end of this function
      
      //Input line
      inputline->keepText (keeptext);
      inputline->selectKeptText (keeptext ? selectkepttext : false);
      inputline->setArrowsHistory (arrowshistory);
      inputline->setAC (useac);
      if (useac)
        inputline->setACType(typeofac);
      inputline->setTelnetPaste(tnpaste);
      cmdparser->setStripSpaces(trimspaces);
      inputline->setPaletteBackgroundColor (color[inputbkcolorindex]);
      inputline->setPaletteForegroundColor (color[inputcolorindex]);
      QPalette p = inputline->palette ();
      p.setColor (QColorGroup::Base, color[inputbkcolorindex]);
      p.setColor (QColorGroup::Text, color[inputcolorindex]);
      inputline->setPalette (p);

      // (auxiliary input line)
      auxinputline->keepText (keeptext);
      auxinputline->selectKeptText (keeptext ? selectkepttext : false);
      auxinputline->setArrowsHistory (arrowshistory);
      auxinputline->setAC (useac);
      if (useac)
        auxinputline->setACType(typeofac);
      auxinputline->setTelnetPaste(tnpaste);
      auxinputline->setPaletteBackgroundColor (color[inputbkcolorindex]);
      auxinputline->setPaletteForegroundColor (color[inputcolorindex]);
      p = auxinputline->palette ();
      p.setColor (QColorGroup::Base, color[inputbkcolorindex]);
      p.setColor (QColorGroup::Text, color[inputcolorindex]);
      auxinputline->setPalette (p);
      
      //multi input line
      multiinputline->keepText (keeptext);
      multiinputline->selectKeptText (keeptext ? selectkepttext : false);
      multiinputline->swapEnters (swapenters);
      multiinputline->setPaletteBackgroundColor (color[inputbkcolorindex]);
      multiinputline->setPaletteForegroundColor (color[inputcolorindex]);
      p = auxinputline->palette ();
      p.setColor (QColorGroup::Base, color[inputbkcolorindex]);
      p.setColor (QColorGroup::Text, color[inputcolorindex]);
      multiinputline->setPalette (p);
      

      //Colors
      for (int i = 0; i < 16; i++)
        ansiparser->setColor(color[i], i);

      //Command echo parameter passed onwards ot cTelnet
      telnet->setCommandEcho(cmdecho);
      
      // LPMUD style prompt handling too...
      telnet->setLPMudStyle (lpmudstyle);

      //Characters (that are no longer single characters)
      cmdparser->setCmdSeparatorString (separstr);
      cmdparser->setSpeedWalkString (walkstr);
      cmdparser->setMacroString (scriptstr);
      cmdparser->setMultiCommandString (multistr);
      cmdparser->setNoParseString (noparsestr);
      cmdparser->setAllowEmptyWalkStr (allowemptywalkstr);
      cmdparser->setExpandBackslashes (expandBS);
      cmdproc->setFocusCommandString (focusstr);

#ifdef HAVE_SOUND
      //MUD Sound Protocol
      telnet->setMSPGlobalPaths (sounddirs);
      telnet->setMSPAllowed(allowmsp);
      telnet->setDownloadAllowed(allowdload);
#endif

#ifdef HAVE_MXP
      mxpmgr->setDefaultFont (cfont, ansiparser->color (fgcolorindex),
          ansiparser->color (bkcolorindex));
#endif

      //Shortcuts
      // --- NOTHING NEEDED ---
    }
  }

  //Window - things that aren't set for every cSession object
  cSessionManager::self()->setAlwaysTabBar (alwaystabbar);
  kmuddy->setAllowGlobalNotify (globalnotify);
  kmuddy->setAllowLocalNotify (localnotify);
  kmuddy->setAlwaysNotify (alwaysnotify);
  kmuddy->setAutoConnect (autoconnect);

  //inform plug-ins
  //this isn't really useful as of now, since we don't allow access to the data, but that will change
  am->invokeEvent ("global-settings-changed", 0);

  //Okay, now we save the settings. This is needed so that other instances
  //can retrieve the new settings...
  save ();
}

void cGlobalSettings::showSettingsDialog ()
{
  //so first we have to create the dialog...
  sdlg = new dlgAppSettings (kmuddy);

  //then we connect() all its signals - this handles everything that the dialog offers...
  connect (sdlg, SIGNAL (okClicked()), this, SLOT (getSettingsFromDialog()));
  connect (sdlg, SIGNAL (applyClicked()), this, SLOT (getSettingsFromDialog()));

  //next we fill in its data
  putSettingsToDialog ();

  //dialog is ready - show it!
  sdlg->exec ();

  //further action is handled via slots issued by buttons, so that we only
  //have to destroy the dialog...
  delete sdlg;
}

void cGlobalSettings::putSettingsToDialog ()
{
  //Window
  sdlg->setAlwaysTabBar (alwaystabbar);
  sdlg->setAuxInput (auxinput);
  sdlg->setAlwaysNotify (alwaysnotify);
  sdlg->setLocalNotify (localnotify);
  sdlg->setGlobalNotify (globalnotify);
  sdlg->setAutoConnect (autoconnect);
  
  //Output area
  sdlg->setcolorindex (0, bkcolorindex);
  sdlg->setcolorindex (1, fgcolorindex);
  sdlg->setcolorindex (2, echoindex);
  sdlg->setcolorindex (3, systemindex);
  sdlg->setEnableBlinking (allowblinking);
  sdlg->setCmdEcho (cmdecho);
  sdlg->setMessages (messages);
  sdlg->setWordWrapping (wordwrapping);
  sdlg->setIndentation (indentvalue);
  sdlg->setWrapPos (wrappos);
  sdlg->setHistory (hissize);
  sdlg->setForceRedraw (forceredraw);
  
  //Fonts
  sdlg->setFont1 (cfont);
  sdlg->setFont2 (ifont);
  sdlg->setFont3 (mfont);
  
  //Input line
  sdlg->setKeepText (keeptext);
  sdlg->setSelectKeptText (selectkepttext);
  sdlg->setArrowsHistory (arrowshistory);
  sdlg->setAutoCompletion (useac);
  sdlg->setACType (typeofac);
  sdlg->setTelnetPaste (tnpaste);
  sdlg->setTrimSpaces (trimspaces);
  sdlg->setInputBkColor (inputbkcolorindex);
  sdlg->setInputColor (inputcolorindex);
  sdlg->setSwapEnters (swapenters);
  
  //Colors
  for (int i = 0; i < 16; i++)
    sdlg->setColor (i, color[i]);
  
  //Characters
  sdlg->setCmdSeparator (separstr);
  sdlg->setSpeedWalk (walkstr);
  sdlg->setScript (scriptstr);
  sdlg->setMultiCmd (multistr);
  sdlg->setFocusCmd (focusstr);
  sdlg->setNoParse (noparsestr);
  sdlg->setAllowEmptyWalkStr (allowemptywalkstr);
  sdlg->setExpandBS (expandBS);
  
#ifdef HAVE_SOUND
  //MSP
  sdlg->setSoundDirs (sounddirs);
  sdlg->setAllowMSP (allowmsp);
  sdlg->setAllowDownloads (allowdload);
#endif
  
  //Shortcuts
  //--- NOTHING NEEDED ---
}

void cGlobalSettings::getSettingsFromDialog ()
{
  //get'em!

  //Window
  alwaystabbar = sdlg->alwaysTabBar ();
  auxinput = sdlg->auxInput ();
  alwaysnotify = sdlg->alwaysNotify ();
  localnotify = sdlg->localNotify ();
  globalnotify = sdlg->globalNotify ();
  autoconnect = sdlg->autoConnect ();
  
  //Output area
  bkcolorindex = sdlg->getcolorindex (0);
  fgcolorindex = sdlg->getcolorindex (1);
  echoindex = sdlg->getcolorindex (2);
  systemindex = sdlg->getcolorindex (3);
  allowblinking = sdlg->enableBlinking ();
  cmdecho = sdlg->cmdEcho ();
  messages = sdlg->messages ();
  wordwrapping = sdlg->wordWrapping ();
  indentvalue = sdlg->indentation ();
  wrappos = sdlg->wrapPos ();
  hissize = sdlg->history ();
  forceredraw = sdlg->forceRedraw ();
  
  //Fonts
  cfont = sdlg->font1 ();
  ifont = sdlg->font2 ();
  mfont = sdlg->font3 ();
  
  //Input line
  keeptext = sdlg->keeptext ();
  selectkepttext = sdlg->selectkepttext ();
  arrowshistory = sdlg->arrowsHistory ();
  useac = sdlg->autocompletion ();
  typeofac = sdlg->actype ();
  tnpaste = sdlg->telnetPaste ();
  trimspaces = sdlg->trimSpaces ();
  inputbkcolorindex = sdlg->inputbkcolor ();
  inputcolorindex = sdlg->inputcolor ();
  swapenters = sdlg->swapEnters ();

  //Colors
  for (int i = 0; i < 16; i++)
    color[i] = sdlg->color (i);

  //Characters
  separstr = sdlg->cmdseparator ();
  walkstr = sdlg->speedwalk ();
  scriptstr = sdlg->script ();
  multistr = sdlg->multicmd ();
  focusstr = sdlg->focus ();
  noparsestr = sdlg->noParse ();
  allowemptywalkstr = sdlg->allowEmptyWalkStr ();
  expandBS = sdlg->expandBS ();
  
#ifdef HAVE_SOUND
  //MSP
  sounddirs = sdlg->soundDirs ();
  allowmsp = sdlg->allowMSP ();
  allowdload = sdlg->allowDownloads ();
#endif
  
  //Shortcuts
  sdlg->applyKeys ();

  //apply'em!
  apply ();
}

