/***************************************************************************
                          cglobalsettings.h  -  global application settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : St Aug 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CGLOBALSETTINGS_H
#define CGLOBALSETTINGS_H

#include <config.h>

#include <qcolor.h>
#include <qfont.h>
#include <qobject.h>
#include <qstringlist.h>

class dlgAppSettings;
class KActionCollection;
/**
Class that stores and manages global application configuration.
  *@author Tomas Mecir
  */

class cGlobalSettings : public QObject  {
  Q_OBJECT
public:
  cGlobalSettings();
  ~cGlobalSettings();
  /** Applies settings. Called when settings change and on application startup. */
  void apply ();
public slots:
  void showSettingsDialog ();
protected slots:
  void getSettingsFromDialog ();
protected:
  /** load settings from kmuddyrc */
  void load ();
  /** save settings there */
  void save ();
  /** put settings to the dialog (called by showSettingsDialog()) */
  void putSettingsToDialog ();
  
  /** the dialog itself */
  dlgAppSettings *sdlg;

//settings data start here...

  //Window
  bool alwaystabbar, auxinput;
  bool globalnotify, localnotify, alwaysnotify;
  QString autoconnect;
  
  //Output area
  int bkcolorindex, fgcolorindex;
  int echoindex;
  int systemindex;
  bool allowblinking;
  bool cmdecho, messages;
  bool lpmudstyle;
  bool wordwrapping;
  int indentvalue, wrappos;
  int hissize;
  int forceredraw;
  
  //Fonts
  QFont cfont, ifont, mfont;
  
  //Input line
  bool keeptext;
  bool selectkepttext;
  bool arrowshistory;
  bool useac;       //ac = auto-completion
  int typeofac;
  bool tnpaste;
  bool trimspaces;
  int inputbkcolorindex, inputcolorindex;
  bool swapenters;
      
  //Colors
  QColor color[16];

  //Characters
  QString separstr, walkstr, scriptstr, multistr, focusstr, noparsestr;
  bool allowemptywalkstr;
   //BS = backslashes
  bool expandBS;
  
#ifdef HAVE_SOUND
  //MUD Sound Prococol settings
  QStringList sounddirs;
  bool allowmsp, allowdload;
#endif
};


#endif
