//
// C++ Implementation: cmultiinputline
//
// Description: 
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cmultiinputline.h"

#include "cansiparser.h"
#include "ccmdqueue.h"
#include "ccmdqueues.h"

#include <kglobalsettings.h>

//maximum number of lines ...
#define MAXLINES 10

cMultiInputLine::cMultiInputLine (int sess, QWidget *parent, const char *name)
 : KTextEdit(parent, name), cActionBase ("multiinputline", sess)
{
  setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed);
  
  setTextFormat (Qt::PlainText);
  
  setWordWrap (QTextEdit::WidgetWidth);
  setWrapPolicy (QTextEdit::AtWordOrDocumentBoundary);
  
  //height: 2 lines
  setLinesHeight (2);
  
  connect (this, SIGNAL (textChanged ()), this, SLOT (updateHeight()));
  
  keeptext = true;
  selectkepttext = true;
  swapenters = false;
}


cMultiInputLine::~cMultiInputLine()
{
}

void cMultiInputLine::initialize ()
{
  //change colors
  cANSIParser *ansiparser = dynamic_cast<cANSIParser *>(object ("ansiparser"));
  
  setPaletteBackgroundColor (ansiparser->color(CL_BLACK));
  setPaletteForegroundColor (ansiparser->color(CL_YELLOW | CL_BRIGHT));
  setBackgroundMode (Qt::FixedColor, Qt::FixedColor);

  //scroll-bars
  setHScrollBarMode (QScrollView::AlwaysOff);
  setVScrollBarMode (QScrollView::AlwaysOn);
  
  //change font
  setFont (KGlobalSettings::fixedFont ()); //default system fixed font
}

void cMultiInputLine::setFont (const QFont &font)
{
  KTextEdit::setFont (font);
  setLinesHeight (_lines);
}

void cMultiInputLine::keepText (bool value)
{
  keeptext = value;
}

void cMultiInputLine::selectKeptText (bool value)
{
  selectkepttext = value;
}

void cMultiInputLine::swapEnters (bool value)
{
  swapenters = value;
}

void cMultiInputLine::setLinesHeight (int lines)
{
  if (lines < 2) lines = 2;
  if (lines > MAXLINES) lines = MAXLINES;
  _lines = lines;
  QFontMetrics fm (font());
  //one pixel space between lines, also one pixel spaces at the top/bottom
  int newheight = lines * (fm.height() + 1) + 2;  
  int frameheight = rect().height() - contentsRect().height();
  setFixedHeight (frameheight + newheight + 1);
}

void cMultiInputLine::updateHeight()
{
  setLinesHeight (lines ());
}

void cMultiInputLine::sendCommands ()
{
  int pars = paragraphs();

  cCmdQueues *queues = (cCmdQueues *) object ("cmdqueues");
  if (!queues) return;
  // create a command queue with all the commands
  cCmdQueue *queue = new cCmdQueue (sess());
  for (int i = 0; i < pars; i++)
    queue->addCommand (text (i));
  queues->addQueue (queue);

  //delete text if not needed
  if (!keeptext)
    setText ("");
  //select everything if needed
  if (selectkepttext)
    selectAll ();
}

void cMultiInputLine::keyPressEvent (QKeyEvent *e)
{
  bool ctrl = false;
  if (e->state() & Qt::ControlButton)
    ctrl = true;
  if ((e->key() == Key_Return) || (e->key() == Key_Enter))
  {
    //ENTER pressed, handle it !
    //we send sommands on plain ENTER, or on CTRL+ENTER if we have to swap them ...
    if (ctrl == swapenters)
      sendCommands ();
    else
      //standard behaviour otherwise
      KTextEdit::keyPressEvent (e);
  }
  else
    KTextEdit::keyPressEvent (e);
}

#include "cmultiinputline.moc"
