/***************************************************************************
                          cmultilineinput.cpp  -  multi line input handler
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po Aug 26 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmultilineinput.h"

#include "cactionmanager.h"
#include "ccmdqueue.h"
#include "ccmdqueues.h"
#include "csessionmanager.h"
#include "dialogs/dlgmultiline.h"

#include "kmuddy.h"
extern KMuddy *kmuddy;

cMultiLineInput::cMultiLineInput ()
{
  multiline = NULL;
  shown = false;  
}

cMultiLineInput::~cMultiLineInput ()
{
  if (multiline != NULL)
    delete multiline;
}

void cMultiLineInput::toggle ()
{
  if (multiline == NULL)
  {
    multiline = new dlgMultiLine (kmuddy);
    connect (multiline, SIGNAL (applyClicked ()), this, SLOT (sendInput ()));
    connect (multiline, SIGNAL (cancelClicked ()), this, SLOT (handleDlgClose ()));
    connect (multiline, SIGNAL (user1Clicked ()), this, SLOT (clearInput ()));
    multiline->setFont (font);
  }

  if (!shown)
  {
    multiline->show ();
    shown = true;
  }
  else
  {
    multiline->hide ();
    shown = false;
    emit dialogClosed ();
  }
}

void cMultiLineInput::handleDlgClose ()
{
  multiline->hide ();
  shown = false;
  emit dialogClosed ();
}

void cMultiLineInput::sendInput ()
{
  cActionManager *am = cActionManager::self();
  int asess = cSessionManager::self()->activeSession();
  cCmdQueues *queues = (cCmdQueues *) am->object ("cmdqueues", asess);
  if (!queues) return;
  // create a command queue with all the commands
  cCmdQueue *queue = new cCmdQueue (asess);
  for (int i = 0; i < multiline->lines(); i++)
    queue->addCommand (multiline->line (i));
  queues->addQueue (queue);
}

void cMultiLineInput::clearInput ()
{
  multiline->clear ();
}

void cMultiLineInput::setFont (QFont _font)
{
  font = _font;
  if (multiline != NULL)
    multiline->setFont (font);
}

