//
// C++ Implementation: cstatusvar
//
// Description: One variable entry in the status bar
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "cstatusvar.h"

#include "cactionmanager.h"
#include "cvariablelist.h"

#include <kconfig.h>

cStatusVar::cStatusVar (int _sess): sess(_sess)
{
  varvalue = maxvarvalue = 0;
  _hidden = false;
  _percentage = false;
}

cStatusVar::~cStatusVar()
{
}

cSaveableField *cStatusVar::newInstance ()
{
  return new cStatusVar (sess);
}

void cStatusVar::updateBar ()
{
  if (_hidden)
    displayedText = QString::null;
  else
  {
    int max = _maxvariable.isEmpty() ? 100 : maxvarvalue;  //max=100 if no variable given
    displayedText = _caption.isEmpty() ? "" : (_caption + " ");
    if (_percentage)
    {
      int percent = (max == 0) ? 0 : (varvalue * 100 / max);
      displayedText += QString::number (percent) + "%";
    }
    else
    {
      displayedText += QString::number (varvalue);
      if (!_maxvariable.isEmpty())
        displayedText += "/" + QString::number (max);
    }
  }
  
  emit textChanged ();
}

void cStatusVar::setVariable (const QString &val)
{
  _variable = val;
  if (!_variable.isEmpty() && _variable[0] == '$')
    _variable = _variable.remove (0, 1);
  varvalue = 0;
  updateBar ();
}

void cStatusVar::setMaxVariable (const QString &val)
{
  _maxvariable = val;
  if (!_maxvariable.isEmpty() && _maxvariable[0] == '$')
    _maxvariable = _variable.remove (0, 1);
  maxvarvalue = 0;
  updateBar ();
}

void cStatusVar::setCaption (const QString &val)
{
  _caption = val;
  updateBar ();
}

void cStatusVar::setPercentage (bool percent)
{
  _percentage = percent;
  updateBar ();
}

void cStatusVar::setHidden (bool h)
{
  bool oldh = _hidden;
  _hidden = h;
  if (h != oldh)
  {
    updateBar ();
  }
}

void cStatusVar::varValueChanged (const QString &var, int newval)
{
  if ((var == _variable) && (varvalue != newval))
  {
    varvalue = newval;
    updateBar ();
  }
  if ((var == _maxvariable) && (maxvarvalue != newval))
  {
    maxvarvalue = newval;
    updateBar ();
  }
}

void cStatusVar::recalc ()
{
  cVariableList *vars = (cVariableList *)
    cActionManager::self()->object ("variablelist", sess);
  if (!vars) return;
  if (_variable)
    varValueChanged (_variable, vars->getIntValue(_variable));
  if (_maxvariable)
    varValueChanged (_maxvariable, vars->getIntValue(_maxvariable));
}

void cStatusVar::load (KConfig *config, const QString &group)
{
  config->setGroup (group);

  setVariable (config->readEntry ("Variable", QString::null));
  setMaxVariable (config->readEntry ("Max variable", QString::null));
  setCaption (config->readEntry ("Caption", QString::null));
  setPercentage (config->readBoolEntry ("Percentage", false));
  setHidden (config->readBoolEntry ("Hidden", false));

  recalc ();
}

void cStatusVar::save (KConfig *config, const QString &group)
{
  config->setGroup (group);

  config->writeEntry ("Variable", _variable);
  config->writeEntry ("Max variable", _maxvariable);
  config->writeEntry ("Caption", _caption);
  config->writeEntry ("Percentage", _percentage);
  config->writeEntry ("Hidden", _hidden);
}


#include "cstatusvar.moc"
