//
// C++ Implementation: cstatusvarlist
//
// Description: 
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cstatusvarlist.h"

#include "cstatus.h"
#include "cstatusvar.h"

#include <kconfig.h>

cStatusVarList::cStatusVarList (QString filename, int sess) :
    cSaveableList ("statusvarlist", sess)
{
  prototype = new cStatusVar (sess);
  objName = "Status-var";
  config = new KConfig (filename);
  load ();
  
  //connect() necessary signals
  for (reset(); *this; (*this)++)
  {
    cStatusVar *sv = (cStatusVar *) cur;
    connect (sv, SIGNAL (textChanged()), this, SLOT (varTextChanged()));
  }
  
  //create the text and send it to the status-bar...
  QString text;
  for (reset(); *this; (*this)++)
  {
    ((cStatusVar *) cur)->recalc ();
    QString t = ((cStatusVar *) cur)->getDisplayedText();
    if (!t.isEmpty())
      text += t + "  ";
  }
  (dynamic_cast<cStatus *>(object ("status")))->displayVariables (text);

  addEventHandler ("var-changed", 500, PT_STRING);
}

cStatusVarList::~cStatusVarList()
{
  removeEventHandler ("var-changed");
  save ();
  config->sync ();
}

void cStatusVarList::eventHandler (QString event, int, QString &par1,
    const QString &par2)
{
  if (event == "var-changed")
    variableChanged (par1, par2);
}

void cStatusVarList::variableChanged (const QString &varname, const QString &varvalue)
{
  bool ok;
  int value = varvalue.toInt (&ok);
  value = ok ? value : 0;
  
  //look if some status-var uses the changed variable
  for (reset(); *this; (*this)++)
  {
    QString v = ((cStatusVar *) cur)->variable();
    QString mv = ((cStatusVar *) cur)->maxVariable();
    //if it does, update it!
    if ((varname == v) || (varname == mv))
      ((cStatusVar *) cur)->varValueChanged (varname, value);
  }
}

void cStatusVarList::varTextChanged ()
{
  //re-create the text and send it to the status-bar...
  QString text;
  for (reset(); *this; (*this)++)
  {
    QString t = ((cStatusVar *) cur)->getDisplayedText();
    if (!t.isEmpty())
      text += t + "  ";
  }
  (dynamic_cast<cStatus *>(object ("status")))->displayVariables (text);
}

bool cStatusVarList::requestStatusVar (const QString &var, const QString &maxvar,
    const QString &caption, bool percentage, bool hidden)
{
  for (reset(); *this; (*this)++)
    if (var == ((cStatusVar *) cur)->variable())  //variable already in use
      return false;
  
  //OKay, variable not yet in use
  cStatusVar *sv = new cStatusVar (sess());
  sv->setVariable (var);
  sv->setMaxVariable (maxvar);
  sv->setCaption (caption);
  sv->setPercentage (percentage);
  sv->setHidden (hidden);
  connect (sv, SIGNAL (textChanged()), this, SLOT (varTextChanged()));
  addToEnd (sv);
  
  return true;
}

bool cStatusVarList::requestStatusVar (cStatusVar *sv)
{
  QString var = sv->variable();
  for (reset(); *this; (*this)++)
    if (var == ((cStatusVar *) cur)->variable())  //variable already in use
    {
      delete sv;  //delete the status-var
      return false;
    }
  
  connect (sv, SIGNAL (textChanged()), this, SLOT (varTextChanged()));
  addToEnd (sv);
  
  return true;
}

