/***************************************************************************
                        ctimertick.cpp  -  description
                             -------------------
    begin                : ?t mj 8 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctimertick.h"

#include "cactionmanager.h"

#include <kconfig.h>

/**
Tick timer.
  *@author Marco Mastroddi
  */

cTimerTick::cTimerTick (int _sess) : cTimer (_sess)
{
  _interval = 75;
  _timeoutmargin = 5;
  _singleshot = false;
  _countdown = -1;
}

cTimerTick::~cTimerTick ()
{
}

cSaveableField *cTimerTick::newInstance ()
{
  return new cTimerTick(sess);
}

QString cTimerTick::performAction ()
{
  if (!_active)
    return QString::null;

  cActionManager::self()->invokeEvent ("command", sess, _command);

  return QString::null;
}

void cTimerTick::load (KConfig *config, const QString &)
{
  // I need to fix the group name to ensure
  // to load always the right preferences
  config->setGroup ("TimerTick");

  setInterval (config->readNumEntry ("Interval", 75));
  setSingleShot (config->readBoolEntry ("Single shot", false));
  setActive (config->readBoolEntry ("Active", false));
  setTimeoutMargin (config->readNumEntry ("Timeout margin", 5));
  setCommand (config->readEntry ("Command", QString( "/echo TICK IN %1 SECONDS.").arg( _timeoutmargin ) ) );
}

void cTimerTick::save (KConfig *config, const QString &)
{
  config->setGroup ("TimerTick");

  config->writeEntry ("Interval", _interval);
  config->writeEntry ("Single shot", _singleshot);
  config->writeEntry ("Active", _active);
  config->writeEntry ("Command", _command);
  config->writeEntry ("Timeout margin", _timeoutmargin);
}

void cTimerTick::tick (int tickTime)
{
  //not active - reset next-launch time and return
  if (!_active)
  {
    _nextlaunch = -1;
    _countdown = -1;
    return;
  }

  if (_interval < 1)  //no interval set
    return;

  if (_nextlaunch == -1) //no next-launch time set
  {
    //update next-launch time and return
    _nextlaunch = tickTime + _interval;
    return;
  }

  _countdown = _nextlaunch - tickTime;

  //check if we should execute now
  if ((_nextlaunch - _timeoutmargin) == tickTime)
  {
    performAction ();
  }

  if (_nextlaunch == tickTime)
  {
    _nextlaunch += _interval;
    //if we're an single-shot timer, we should be deleted
    if (_singleshot)
      _shoulddelete = true;
  }
}


