//
// C++ Implementation: cvartriggerlist
//
// Description:
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cvartriggerlist.h"

#include "cactionmanager.h"
#include "cvartrigger.h"

#include <kconfig.h>

cVarTriggerList::cVarTriggerList (QString filename, int sess)
  : cSaveableList ("vartriggerlist", sess)
{
  prototype = new cVarTrigger (sess);
  objName = "Variable trigger";
  config = new KConfig (filename);
  load ();

  addEventHandler ("old-var-value", 50, PT_STRING);
  addEventHandler ("var-changed", 50, PT_STRING);
}


cVarTriggerList::~cVarTriggerList()
{
  removeEventHandler ("var-changed");
  removeEventHandler ("old-var-value");
  save ();
  config->sync ();
}

void cVarTriggerList::eventHandler (QString event, int session,
    QString &par1, const QString &par2)
{
  static QString lastvar;
  if (event == "old-var-value")
    lastvar = par2;
  if (event == "var-changed")
    if (par2 != lastvar)
      execTriggers (par1, par2);
}

void cVarTriggerList::execTriggers (const QString &varname, const QString &)
{
  //FIXME: this is SLOW!!! Use multimap (in the class) instead of list traversing!!!

  for (reset(); *this; (*this)++)
  {
    cVarTrigger *vt = (cVarTrigger *) cur;
    if (vt->varName () == varname)
    {
      vt->performAction ();
      processCommands (vt->commandsToExec());
    }
  }
}

void cVarTriggerList::processCommands (const QStringList &commands)
{
  QStringList::const_iterator it;
  for (it = commands.begin(); it != commands.end(); ++it)
    // command coming from a trigger (some processing is different then)
    cActionManager::self()->invokeEvent ("command", sess(), *it, "trigger");
}

