//
// C++ Implementation: dlgeditstatusvar
//
// Description: Edit status variable dialog
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "dlgeditstatusvar.h"

#include "cstatusvar.h"

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qwhatsthis.h>

#include <klineedit.h>
#include <klocale.h>

dlgEditStatusVar::dlgEditStatusVar (cStatusVar *sv, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Edit status variable"), Ok|Cancel, Ok)
{
  statusvar = sv;
  
  createDialog ();
}


dlgEditStatusVar::~dlgEditStatusVar()
{
}

void dlgEditStatusVar::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (400, 200));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 6, 2);

  setMainWidget (page);

  //variable
  QLabel *lbl1 = new QLabel (i18n ("&Variable"), page);
  variable = new KLineEdit (page);
  lbl1->setBuddy (variable);
  QWhatsThis::add (variable, i18n ("Variable whose value shall be displayed."));

  //max. variable
  QLabel *lbl2 = new QLabel (i18n ("&Max. variable (optional)"), page);
  maxvariable = new KLineEdit (page);
  lbl2->setBuddy (maxvariable);
  QWhatsThis::add (maxvariable, i18n ("Variable holding maximum value (default=100)"));
  
  //caption
  QLabel *lbl3 = new QLabel (i18n ("&Caption"), page);
  caption = new KLineEdit (page);
  lbl3->setBuddy (caption);
  QWhatsThis::add (caption, i18n ("Caption shown next to variable value."));

  //percentage
  chkpercent = new QCheckBox (i18n ("&Show as percentage"), page);
  QWhatsThis::add (chkpercent, i18n ("When enabled, this variable will be shown as percentage. "
      "(of max.value, or of 100 if no max.value given)"));
  
  //hidden
  chkhidden = new QCheckBox (i18n ("&Hidden"), page);
  QWhatsThis::add (chkhidden, i18n ("If checked, then this status variable won't be shown."));

  //place'em there!
  layout->setSpacing (5);
  layout->addWidget (lbl1, 0, 0);
  layout->addWidget (variable, 0, 1);
  layout->addWidget (lbl2, 1, 0);
  layout->addWidget (maxvariable, 1, 1);
  layout->addWidget (lbl3, 2, 0);
  layout->addWidget (caption, 2, 1);
  layout->addMultiCellWidget (chkpercent, 3, 3, 0, 1);
  layout->addMultiCellWidget (chkhidden, 4, 4, 0, 1);
  layout->setRowStretch (5, 10);
}

bool dlgEditStatusVar::showMe ()
//fills and shows the dialog, then applies changes if needed
{
  //fill in the dialog
  variable->setText (statusvar->variable ());
  maxvariable->setText (statusvar->maxVariable ());
  caption->setText (statusvar->caption ());
  chkpercent->setChecked (statusvar->percentage ());
  chkhidden->setChecked (statusvar->hidden ());
  
  //show it!
  int n = exec ();

  //update structure if needed
  if (n == Accepted)
  {
    statusvar->setVariable (variable->text ());
    statusvar->setMaxVariable (maxvariable->text ());
    statusvar->setCaption (caption->text ());
    statusvar->setPercentage (chkpercent->isChecked ());
    statusvar->setHidden (chkhidden->isChecked ());
    return true;
  }
  else
    return false;
}


#include "dlgeditstatusvar.moc"
