/***************************************************************************
                          dlgstatistics.cpp  -  Statistics dialog
                             -------------------
    begin                : Pi mar 14 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgstatistics.h"

#include "cactionmanager.h"
#include "cconnection.h"
#include "csessionmanager.h"
#include "cstatus.h"
#include "ctelnet.h"
#include "ctextprocessor.h"

#include <config.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>
#include <klistview.h>
#include <klocale.h>

dlgStatistics::dlgStatistics (QWidget *parent, const char *name ) :
    KDialogBase (parent, name, false, i18n ("Connection statistics"), 0)
{
  //initial size
  setInitialSize (QSize (250, 300));

  //main widget
  QWidget *page = new QWidget (this);
  QVBoxLayout *layout = new QVBoxLayout (page);

  setMainWidget (page);

  setFocusPolicy (QWidget::StrongFocus);

  QLabel *label = new QLabel (i18n ("&Statistics, updated every second:"), page);
  view = new KListView (page);
  label->setBuddy (view);
  view->addColumn (i18n ("Counter"));
  view->addColumn (i18n ("Value"));
  view->setColumnWidthMode (0, QListView::Maximum);
  view->setColumnWidthMode (1, QListView::Maximum);
  view->setSorting (-1);
  view->setSelectionMode (QListView::NoSelection);
  
  layout->setSpacing (5);
  layout->addWidget (label);
  layout->addWidget (view);

  //establish auto-updating of data
  updateTimer = new QTimer;
  connect (updateTimer, SIGNAL (timeout()), this, SLOT (update()));
  updateTimer->start (1000);
  
  //update the view!
  update ();  
}

dlgStatistics::~dlgStatistics ()
{
  updateTimer->stop ();
  delete updateTimer;
}

void dlgStatistics::update ()
{
  view->clear ();

  view->setSelectionMode (QListView::NoSelection);

  //fetch required information
  int sess = cSessionManager::self()->activeSession();
  cActionManager *am = cActionManager::self();
  cTelnet *telnet = dynamic_cast<cTelnet *>(am->object ("telnet", sess));
  cConnection *connection = dynamic_cast<cConnection *>(am->object ("connection", sess));
  cStatus *status = dynamic_cast<cStatus *>(am->object ("status", sess));
  cTextProcessor *textproc = dynamic_cast<cTextProcessor *>(am->object ("textproc", sess));
  int compr = telnet->compressedBytes();
  int decompr = telnet->uncompressedBytes();
  QString connTime = status->connTimeString();
  int sentbytes = telnet->sentBytes();
  int sentcmds = connection->sentCommands();
  int recvlines = textproc->linesReceived();
  bool usingmccp = telnet->usingMCCP();
  int mccpver = telnet->MCCPVer();
#ifdef HAVE_SOUND
  bool usingmsp = telnet->usingMSP();
#endif
  
  //fill in those information

  //pointer is used to store last item; by default, new items are added to
  //the top of the view, which is wrong
  QListViewItem *pointer = 0;
  pointer = new QListViewItem (view, pointer, i18n ("Using MCCP"),
      usingmccp ? i18n ("Yes") : i18n ("No"));
  pointer = new QListViewItem (view, pointer, i18n ("MCCP version"),
      usingmccp ? QString::number (mccpver) : i18n ("N/A"));
#ifdef HAVE_SOUND
  pointer = new QListViewItem (view, pointer, i18n ("Using MSP"),
      usingmsp ? i18n ("Yes") : i18n ("No"));
#endif
  pointer = new QListViewItem (view, pointer, i18n ("Sent bytes"),
      QString::number (sentbytes));
  pointer = new QListViewItem (view, pointer, i18n ("Sent commands"),
      QString::number (sentcmds));
  pointer = new QListViewItem (view, pointer, i18n ("Received bytes"),
      QString::number (compr));
  pointer = new QListViewItem (view, pointer, i18n ("... after decompression"),
      QString::number (decompr));
  pointer = new QListViewItem (view, pointer, i18n ("Received lines"),
      QString::number (recvlines));
  pointer = new QListViewItem (view, pointer, i18n ("Connection time"),
      connTime);

  view->triggerUpdate ();
}

