/***************************************************************************
                        dlgtimertick.cpp  -  description
                             -------------------
    begin                : Wed Feb 12, 2004
    copyright            : (C) 2004 by Marco Mastroddi
    email                : marco.mastroddi@libero.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgtimertick.h"
#include "ctimerlist.h"
#include "ctimertick.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/**
Tick timer dialog.
  *@author Marco Mastroddi
  */


dlgTimerTick::dlgTimerTick( cTimerTick *tt, QWidget* parent, const char* name )
    : QDialog( parent, name, true )
{
  timerTick = tt;
  resetTickTimer = false;

  if ( !name )
    setName( "dlgTimerTick" );

  dlgTimerTickLayout = new QGridLayout( this, 1, 1, 11, 6, "dlgTimerTickLayout");
  QSpacerItem* spacer = new QSpacerItem( 90, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
  dlgTimerTickLayout->addItem( spacer, 2, 0 );

  bOk = new QPushButton( this, "bOk" );
  dlgTimerTickLayout->addWidget( bOk, 2, 1 );
  bCancel = new QPushButton( this, "bCancel" );
  dlgTimerTickLayout->addWidget( bCancel, 2, 2 );

  cbEnableTickTimer = new QCheckBox( this, "cbEnableTickTimer" );
  dlgTimerTickLayout->addMultiCellWidget( cbEnableTickTimer, 0, 0, 0, 2 );

  frameTick = new QFrame( this, "frameTick" );
  frameTick->setFrameShape( QFrame::Box );
  frameTick->setFrameShadow( QFrame::Sunken );
  frameTickLayout = new QGridLayout( frameTick, 1, 1, 11, 6, "frameTickLayout");

  label3 = new QLabel( frameTick, "label3" );
  frameTickLayout->addWidget( label3, 2, 0 );

  leCommand = new QLineEdit( frameTick, "leCommand" );
  leCommand->setMinimumSize( QSize( 160, 0 ) );
  frameTickLayout->addMultiCellWidget( leCommand, 2, 2, 1, 2 );

  label2 = new QLabel( frameTick, "label2" );
  frameTickLayout->addWidget( label2, 1, 0 );

  sbTimeoutMargin = new QSpinBox( frameTick, "sbTimeoutMargin" );
  sbTimeoutMargin->setMaxValue( 20 );
  sbTimeoutMargin->setValue( 5 );
  frameTickLayout->addWidget( sbTimeoutMargin, 1, 1 );
  QSpacerItem* spacer_2 = new QSpacerItem( 80, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
  frameTickLayout->addItem( spacer_2, 1, 2 );
  QSpacerItem* spacer_3 = new QSpacerItem( 80, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
  frameTickLayout->addItem( spacer_3, 0, 2 );

  sbTickInterval = new QSpinBox( frameTick, "sbTickInterval" );
  sbTickInterval->setMaxValue( 999 );
  sbTickInterval->setMinValue( 25 );
  sbTickInterval->setValue( 75 );
  frameTickLayout->addWidget( sbTickInterval, 0, 1 );

  label1 = new QLabel( frameTick, "label1" );
  frameTickLayout->addWidget( label1, 0, 0 );

  dlgTimerTickLayout->addMultiCellWidget( frameTick, 1, 1, 0, 2 );
  languageChange();
  resize( QSize(360, 191).expandedTo(minimumSizeHint()) );
  clearWState( WState_Polished );

  // tab order
  setTabOrder( cbEnableTickTimer, sbTickInterval );
  setTabOrder( sbTickInterval, sbTimeoutMargin );
  setTabOrder( sbTimeoutMargin, leCommand );
  setTabOrder( leCommand, bOk );
  setTabOrder( bOk, bCancel );

  connect( cbEnableTickTimer, SIGNAL( stateChanged( int ) ), this, SLOT( somethingChanged() ) );
  connect( sbTickInterval, SIGNAL( valueChanged( int ) ), this, SLOT( somethingChanged() ) );
  connect( bOk, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
  connect( bCancel, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
  updateMenu();
}

dlgTimerTick::~dlgTimerTick()
{
  // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void dlgTimerTick::languageChange()
{
  setCaption( tr( "Tick Timer" ) );
  bOk->setText( tr( "&Ok" ) );
  bOk->setAccel( QKeySequence( tr( "Alt+O" ) ) );
  bCancel->setText( tr( "&Cancel" ) );
  bCancel->setAccel( QKeySequence( tr( "Alt+C" ) ) );
  cbEnableTickTimer->setText( tr( "Enable Tick Timer" ) );
  label3->setText( tr( "Command" ) );
  label2->setText( tr( "Timeout margin" ) );
  label1->setText( tr( "Tick interval" ) );
}

void dlgTimerTick::updateMenu()
{
  cbEnableTickTimer->setChecked( timerTick->active() );
  sbTickInterval->setValue( timerTick->interval() );
  sbTimeoutMargin->setValue( timerTick->timeoutMargin() );
  leCommand->setText( timerTick->command() );
}

void dlgTimerTick::slotOk()
{
  timerTick->setActive( cbEnableTickTimer->isChecked() );
  timerTick->setInterval( sbTickInterval->value() );
  timerTick->setTimeoutMargin( sbTimeoutMargin->value() );
  timerTick->setCommand( leCommand->text() );
  if( resetTickTimer )
    timerTick->resetTickCount();
  done(1);
}

void dlgTimerTick::slotCancel()
{
  done(0);
}

void dlgTimerTick::somethingChanged()
{
  // We need to restart timer tick
  // if the user change some parameters.
  resetTickTimer = true;
}
