/***************************************************************************
                          caliaslist.cpp  -  alias list
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ct sep 12 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "caliaslist.h"

#include "calias.h"
#include "cexpresolver.h"

#include <kconfig.h>

cAliasList::cAliasList (QString filename, int sess) :
    cSaveableList ("aliaslist", sess)
{
  //sess is assigned in cSaveableList's constructor
  prototype = new cAlias (sess);
  objName = "Alias";
  config = new KConfig (filename);
  load ();
  resolver = new cExpResolver (sess);
}

cAliasList::~cAliasList ()
{
  delete resolver;
  save ();
  config->sync ();
}

void cAliasList::newString ()
{
  wasmatch = false;
  matchedcommands.clear();
}

void cAliasList::matchString (const QString &string)
{
  if (_enabled) //only if processing is enabled
  {
    //cycle through all items in the list...
    for (reset(); *this; (*this)++)
    {
      if (!_enabled)
        break;
      unsigned int mpos = 0;  // reset the index where matching will start
      //match against alias
      bool everMatched = false;
      while (true) {
        bool matched, condmatched = true;
        cur->setMatchingPos (mpos);  // set position of matching
        matched = cur->match (string);
        if (matched)
        {
          condmatched = cur->testCondition (sess(), resolver);
          if (condmatched)
          {
            cur->performAction ();
            processCommands (cur->commandsToExec ());
            everMatched = true;   // we have at least one match now
          }
        }
        if ((!matched) || (!cur->global()))
          break;  // not matched or not global - no further repeats
        
        // global - update matching position, or terminate if no more matching should occur
        // if last length is 0, we must advance by 1 to avoid an endless loop ...
        int shift = (cur->getLastLength() == 0) ? 1 : cur->getLastLength();
        mpos = cur->getLastPos() + shift;
        if (mpos >= string.length())
          break;
      }
      //break the loop if needed
      if (!(cur->shouldContinue (everMatched)))
        break;
      //also break if we're in recursion
      if (recursionAlert)
        break;
    }
    reset ();
  }
}

void cAliasList::processCommands (const QStringList &commands)
{
  if (!wasmatch)
    matchedcommands.clear ();
  wasmatch = true;
  matchedcommands += commands;
}
