/***************************************************************************
                          cstatus.cpp -  manages status bar
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne Jul 7 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cstatus.h"

#include <qdatetime.h>
#include <klocale.h>

#include "cconnprefs.h"

#define ID_DIMENSION 1
#define ID_TIMER 2
#define ID_IDLE 3
#define ID_CONNECTED 5
#define ID_VARIABLES 10
#define ID_PARTIAL 11

cStatus::cStatus (int sess, KStatusBar *statusbar)
  : cActionBase ("status", sess)
{
  sb = statusbar;

  sb->insertItem (" " + i18n ("Off-line") + " ", ID_CONNECTED, 0, true);
  sb->insertItem (" ??x?? ", ID_DIMENSION, 0, true);
  sb->insertItem (" 0:00:00 ", ID_TIMER, 0, true);
  sb->insertItem (" " + i18n ("idle") + " 0:00 ", ID_IDLE, 0, true);
  sb->insertItem ("", ID_VARIABLES);
  sb->insertItem ("", ID_PARTIAL);
  timerShown = true;

  //dimension will be signaled soon - don't bother with it now...

  timing = false;
  timer = new QTimer;
  timer1 = new QTimer;
  connect (timer, SIGNAL (timeout ()), this, SLOT (timerTick ()));
  connect (timer1, SIGNAL (timeout ()), this, SLOT (timer1Tick ()));
  timer->start (1000);

  timerTick ();

  addEventHandler ("connected", 200, PT_NOTHING);
  addEventHandler ("disconnected", 200, PT_NOTHING);
  addEventHandler ("got-prompt", 100, PT_STRING);
  addEventHandler ("partial-line", 100, PT_STRING);
  addEventHandler ("command-sent", 100, PT_STRING);
  addEventHandler ("dimensions-changed", 100, PT_INT);
}

cStatus::~cStatus()
{
  removeEventHandler ("connected");
  removeEventHandler ("disconnected");
  removeEventHandler ("got-prompt");
  removeEventHandler ("partial-line");
  removeEventHandler ("command-sent");
  removeEventHandler ("dimensions-changed");
  
  timer->stop ();
  timer1->stop ();
  delete timer;
  delete timer1;
  
  delete sb;
}

void cStatus::eventHandler (QString event, int)
{
  if (event == "connected") {
    connected ();
    timerReset ();
  }
  else if (event == "disconnected") {
    disconnected ();
    timerStop ();
  }
}

void cStatus::eventHandler (QString event, int, QString &par1, const QString &)
{
  if (event == "got-prompt") {
    partialLine (par1);
  }
  else if (event == "partial-line") {
    partialLine (par1);
  }
  else if (event == "command-sent") {
    gotCommand ();
  }
}

void cStatus::eventHandler (QString event, int, int par1, int par2)
{
  if (event == "dimensions-changed") {
    dimensionsChanged (par1, par2);
  }
}

void cStatus::showTimer ()
{
  if (!timerShown)
  {
    timerReset ();
    sb->changeItem (" 0:00:00 ", ID_TIMER);
  }
  timerShown = true;
}

void cStatus::hideTimer ()
{
  if (timerShown)
  {
    timerStop ();
    sb->changeItem ("", ID_TIMER);  //item is invisible when its text is empty
  }
  timerShown = false;
}

void cStatus::showMessage (const QString & message)
{
  sb->message (message, 2000);
}

void cStatus::dimensionsChanged (int x, int y)
{
  char s1[10];
  QString s2;
  sprintf (s1, " %dx%d ", x, y);
  s2 = s1;
  sb->changeItem ("xxxxx", ID_DIMENSION);
  sb->changeItem (s2, ID_DIMENSION);
  sb->repaint ();
}

void cStatus::timerStart ()
{
  timing = true;
}

void cStatus::timerStop ()
{
  timing = false;

  timer1->stop ();
  sb->changeItem ("", ID_IDLE);
}

void cStatus::timerReset ()
{
  timing = true;
  conntime = 0;
  idletime1 = 0;
  if (timerShown)
    sb->changeItem (" 0:00:00 ", ID_TIMER);
  sb->changeItem (" " + i18n ("idle") + " 0:00", ID_IDLE);
  timer1->start (1000);
}

void cStatus::connected ()
{
  sb->changeItem (" " + i18n ("Connected") + " ", ID_CONNECTED);
  sb->changeItem ("", ID_PARTIAL);
  sb->changeItem (" " + i18n ("idle") + " 0:00 ", ID_IDLE);
  showMessage (i18n ("Connected."));
}

void cStatus::disconnected ()
{
  sb->changeItem (" " + i18n ("Off-line") + " ", ID_CONNECTED);
  showMessage (i18n ("Disconnected."));
  sb->changeItem ("", ID_PARTIAL);
  invokeEvent ("message", sess(), i18n ("Connection has been closed."));
  timerStop ();
}

void cStatus::partialLine (const QString &line)
{
  cConnPrefs *cp = dynamic_cast<cConnPrefs *>(object ("connprefs"));
  //partial line displayed if enabled in profile prefs, or if this is a quick
  //connection
  if ((!cp) || cp->statusPrompt())
    sb->changeItem (line, ID_PARTIAL);
}

void cStatus::clearPartialLine ()
{
  sb->changeItem ("", ID_PARTIAL);
}

void cStatus::gotCommand ()
{
  if (timing)
  {
    idletime1 = 0;
    timer1->stop ();
    timer1->start (1000);
    sb->changeItem (" " + i18n ("idle") + " 0:00 ", ID_IDLE);
  }
}

const QString cStatus::connTimeString ()
{
  char s1[10];
  int h = conntime / 3600;
  int s = conntime % 3600;
  int m = s / 60;
  s = s % 60;
  sprintf (s1, " %d:%02d:%02d ", h, m, s);
  QString s2 = s1;
  return s2;
}

void cStatus::displayVariables (const QString varText)
{
  sb->changeItem (varText, ID_VARIABLES);
}

void cStatus::timerTick ()
{
  QString s2;
  if (timing)
  {
    conntime++;

    s2 = connTimeString ();
    sb->changeItem (s2, ID_TIMER);
  }
}

void cStatus::timer1Tick ()
{
  ++idletime1;
  
  int h, m, s;
  s = idletime1 % 60;
  m = ((idletime1 - s) / 60) % 60;
  h = (idletime1 - s) / 3600;
  
  char ss[15];
  if (h > 0)
    sprintf (ss, " %d:%02d:%02d ", h, m, s);
  else
    sprintf (ss, " %d:%02d ", m, s);

  sb->changeItem (" " + i18n ("idle") + ss, ID_IDLE);
}


