//
// C++ Implementation: scriptingplugin
//
// Description: scriptingplugin
//
//
// Author: YOUR-NAME, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "scriptingplugin.h"

#include "cactionmanager.h"
#include "cconnprefs.h"
#include "cmacromanager.h"
#include "cmenumanager.h"

#include "cnotifymanager.h"
#include "crunninglist.h"
#include "cscriptlist.h"
#include "dlgrunninglist.h"
#include "dlgscripts.h"

#include <kaction.h>
#include <kapplication.h>
#include <kgenericfactory.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include <map>

typedef KGenericFactory<cScriptingPlugin> cScriptingPluginFactory;
K_EXPORT_COMPONENT_FACTORY (kmuddyscriptingplugin, cScriptingPluginFactory("kmuddy"))


// Macros exec and notify:

class cMacroExec : public cMacro {
  public:
    cMacroExec () : cMacro ("exec") {}
    virtual void eval (const QString &params, int sess, cCmdQueue *queue) {
      QString sname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
      QString pars = params.section (' ', 1, -1, QString::SectionSkipEmpty);
      pars = expandVariables (pars, sess, queue);
      cScriptList *list = dynamic_cast<cScriptList *>(am->object ("scriptlist", sess));

// Run it!!!  This will also display error message if something
// goes wrong.
      list->runScript (sname, pars);
    }
};

class cMacroNotify : public cMacro {
  public:
    cMacroNotify () : cMacro ("notify") {
      notifymanager = new cNotifyManager;
    }
    ~cMacroNotify () {
      delete notifymanager;
    }
    virtual void eval (const QString &params, int sess, cCmdQueue *queue) {
      QString pars = expandVariables (params, sess, queue);
      QString ip_port = pars.section (' ', 0, 0, QString::SectionSkipEmpty);
      QString ip_data = pars.section (' ', 1, -1, QString::SectionSkipEmpty);

      bool convert_ok;
      int  port;
      port = ip_port.toInt(&convert_ok);
// Send data to IP port
      if (convert_ok)
        notifymanager->doNotify (port, ip_data);
    }
  private:
    cNotifyManager *notifymanager;
};



struct ScriptingSessionData {
  cScriptList *scripts;
  cRunningList *runningList;
};

struct cScriptingPluginPrivate {
  KToggleAction *showrunningscripts;
  KAction *scripts;
  dlgRunningList *rdlg;

  cMacroExec *mexec;
  cMacroNotify *mnotify;
  
  std::map<int, ScriptingSessionData> sessionData;
};

cScriptingPlugin::cScriptingPlugin (QObject *, const char *, const QStringList &)
{
  setInstance (cScriptingPluginFactory::instance());
  
  d = new cScriptingPluginPrivate;
  d->rdlg = 0;

  d->mexec = new cMacroExec;
  d->mnotify = new cMacroNotify;

  KActionCollection *acol = cActionManager::self()->getACol ();
  d->scripts = new KAction (i18n ("&Scripts..."), KShortcut::null(), this,
      SLOT (handleScriptsDialog()), acol, "Scripts");
  d->showrunningscripts = new KToggleAction (i18n ("Show &running scripts"), KShortcut::null(),
    acol, "ShowRunningScripts");
  connect (d->showrunningscripts, SIGNAL (toggled (bool)), this,
      SLOT (showRunningScripts (bool)));
  d->showrunningscripts->setChecked (false);

  enableMenus (false);

  cMenuManager *menu = cMenuManager::self();
  menu->plug (d->showrunningscripts, "view-profile");
  menu->plug (d->scripts, "profile-objects");
}

cScriptingPlugin::~cScriptingPlugin()
{
  cMenuManager *menu = cMenuManager::self();
  menu->unplug (d->showrunningscripts);
  menu->unplug (d->scripts);
  delete d->scripts;
  delete d->showrunningscripts;
  delete d->rdlg;
  delete d->mexec;
  delete d->mnotify;
  delete d;
}

void cScriptingPlugin::sessionAdd (int sess, bool)
{
  ScriptingSessionData sd;
  sd.scripts = 0;
  sd.runningList = 0;
  d->sessionData[sess] = sd;
}

void cScriptingPlugin::sessionRemove (int sess, bool)
{
  if (d->sessionData.count (sess)) {
    delete d->sessionData[sess].scripts;
    delete d->sessionData[sess].runningList;
  }
  d->sessionData.erase (sess);
}

void cScriptingPlugin::sessionSwitch (int sess)
{
  updateRunningList ();
  bool profile = (cActionManager::self()->sessionAttrib (sess, "profile") != 0);
  enableMenus (profile);
}

void cScriptingPlugin::connected (int sess)
{
  if (cActionManager::self()->activeSession() == sess)
    sessionSwitch (sess);
}

void cScriptingPlugin::disconnected (int sess)
{
  if (d->sessionData.count (sess)) {
    delete d->sessionData[sess].scripts;
    d->sessionData[sess].scripts = 0;
    delete d->sessionData[sess].runningList;
    d->sessionData[sess].runningList = 0;
  }

  if (cActionManager::self()->activeSession() == sess)
    updateRunningList ();
}

void cScriptingPlugin::load (int sess)
{
  if (!d->sessionData.count (sess))
    return;
  
  cActionManager *am = cActionManager::self();
  bool profile = (am->sessionAttrib (sess, "profile") != 0);
  if (!profile) {
    d->sessionData[sess].scripts = 0; 
    d->sessionData[sess].runningList = 0; 
    return;
  }
  
  cConnPrefs *cp = dynamic_cast<cConnPrefs *>(am->object ("connprefs", sess));
  if (!cp) return;  // should never happen
  
  // create and load script lists
  QString directory = locateLocal ("appdata", "profiles/" + cp->name() + "/");
  d->sessionData[sess].scripts = new cScriptList (directory + "scripts", sess);
  d->sessionData[sess].runningList = new cRunningList (sess);
}

void cScriptingPlugin::save (int sess)
{
  if (!d->sessionData.count (sess)) return;
  if (!d->sessionData[sess].scripts) return;
  
  d->sessionData[sess].scripts->save();
}

void cScriptingPlugin::handleScriptsDialog ()
{
  cActionManager *am = cActionManager::self();
  int sess = am->activeSession();
  cScriptList *sl = dynamic_cast<cScriptList *>(am->object ("scriptlist", sess));
  if (!sl) return;

  //so first we have to create the dialog...
  dlgScripts *scdlg = new dlgScripts (sl, KApplication::kApplication()->mainWidget ());

  //dialog is ready - show it!
  scdlg->exec ();

  //further action is handled by the dialog itself, so that we only
  //have to destroy the dialog...
  delete scdlg;
  
  cActionManager::self()->invokeEvent ("save", cActionManager::self()->activeSession());
}

void cScriptingPlugin::showRunningScripts (bool val)
{
  cActionManager *am = cActionManager::self();
  cRunningList *list;
  list = dynamic_cast<cRunningList *>(am->object ("runninglist", activeSession()));

  if (d->rdlg == 0)
  {
    d->rdlg = new dlgRunningList (list, KApplication::kApplication()->mainWidget ());
    connect (d->rdlg, SIGNAL (finished()), this, SLOT (runningScriptsFinished ()));
  }
  if (val)
    d->rdlg->show ();
  else
    d->rdlg->hide ();

  //keep action in sync (this function can be explicitly called)
  d->showrunningscripts->setChecked (val);
}

void cScriptingPlugin::runningScriptsFinished ()
{
  showRunningScripts (false);
}

void cScriptingPlugin::updateRunningList ()
{
  cActionManager *am = cActionManager::self();
  int s = am->activeSession();
  cRunningList *rl = dynamic_cast<cRunningList *>(am->object ("runninglist", s));
  if (d->rdlg != 0)
    d->rdlg->switchRunningList (rl);
}

void cScriptingPlugin::enableMenus (bool enable)
{
  d->scripts->setEnabled (enable);
}

#include "scriptingplugin.moc"
